<?php

/*
Plugin Name: Goal Tracker for Patreon
Description: This plugin connects to your Patreon account and allows you to show progress toward your goals.
Version: 0.4.6
Author: Jerry Seeger
License: GPLv2 or newer
License URI: http://www.gnu.org/licenses/gpl-2.0.html
*/

namespace GoalTrackerPatreon;

defined( 'ABSPATH' ) or die;

$pluginDir = plugin_dir_path(__FILE__);

require_once($pluginDir.'lib/Options.php');
require_once($pluginDir.'lib/Creator.php');
require_once($pluginDir.'lib/Goal_widget.php');
require_once($pluginDir.'lib/Button_widget.php');

class Tracker {

	public function __construct() {
		add_action('widgets_init', array($this, 'registerWidgets'));
		add_action('admin_init', array($this, 'checkSettings'));
	}
	
	public function registerWidgets() {
		register_widget( '\GoalTrackerPatreon\Goal_tracker_widget' );
		register_widget( '\GoalTrackerPatreon\Button_widget' );
	}

	function checkSettings() {
		$cs = get_option(Options::client_secret_option_name);
		$cid = get_option(Options::client_id_option_name);
		$rt = get_option(Options::refresh_token_option_name);
		$at = get_option(Options::access_token_option_name);
	
		$allSet = !empty($cs) && !empty($cid) && !empty($rt) && !empty($at);
	
		if (!$allSet) {
			add_action('admin_notices', array($this, 'showConfigAlert'));
		}
	}

	function showConfigAlert() {
		$message = 'Goal Tracker for Patreon needs information to connect to Patreon and to automatically renew your Patreon access token. Please visit the <a href="'.site_url().'/wp-admin/options-general.php?page=goals_for_patreon_options">options page</a> and add the needed items.';
		$html = <<<HTML
			<div class="notice update-nag">
				<p>$message</p>
			</div>
HTML;

		print $html;
	}

}

new Tracker();
