/**
 * Created by Jerry on 4/1/16.
 */

window.patreonCampaignTracker = {

	setTextColorFromWidget: function (widgetId) {
		var widget = jQuery('#'+widgetId);
		var color = widget.css('color');
		widget.find('text').css('fill', color);
	},
	
	setTickColorFromText: function (widgetId) {
		var widget = jQuery('#'+widgetId);
		var color = widget.find('text').css('fill');
		widget.find('line').css('stroke', color);
	},
	
	loadGoal: function (siteURL, widgetId, goalId, startCents) {
		var widget = document.getElementById(widgetId);
		var widgetWidth = jQuery(widget).width();

		widget.querySelector('svg').setAttribute('width', widgetWidth);

		var stretchWidth = widgetWidth - 40;
		widget.querySelector('rect.thermometer-background').setAttribute('width', stretchWidth+2);
		widget.querySelector('g.goal-tick').setAttribute('transform', 'translate(' + stretchWidth + ', 13)');

		var self = this;
		
		jQuery.ajax({
			url: siteURL + '/wp-admin/admin-ajax.php',
			data: {action: 'get_goal', goalId: goalId},
			dataType: 'json',
			success: function (data) {
				if (data.error) {
					widget.querySelector('.patreon-campaign-info').innerHTML = '<span class="patreon-error-message">Error contacting Patreon: '+data.error+'</span>';
				} else {
					var goal = data.goal.attributes;
					var campaign = data.campaign.attributes;
							
					var scale = stretchWidth / (goal.amount - startCents);
					var progress = Math.min(Math.max(campaign.pledge_sum - startCents, 0), goal.amount);
					var progressWidth = scale * progress;

					var summary = 'Pledged so far: $' + campaign.pledge_sum / 100 + ' per ' + campaign.pay_per_name;
					summary += '<br>Pledge goal: $' + (goal.amount / 100 + ' per ' + campaign.pay_per_name);
					widget.querySelector('.patreon-campaign-info').innerHTML = summary;
					widget.querySelector('.goal-tick-label tspan').textContent = goal.amount / 100;
					widget.querySelector('g.mercury').setAttribute('transform', 'scale('+progressWidth+', 1)');

					var trackerLabel = widget.querySelector('g.tracker-tick text');
					trackerLabel.querySelector('tspan').textContent = Math.max(campaign.pledge_sum, startCents) / 100;
					var labelWidth = trackerLabel.getComputedTextLength();
					widget.querySelector('.tracker-tick line.horizontal').setAttribute('x1', '-' + (labelWidth + 2));
					
					self.buildStyleSheet(widget, widgetId, progressWidth);
					
					var thingsThatMove = [widget.querySelector('rect.mercury'), widget.querySelector('g.tracker-tick')];
					thingsThatMove.forEach(function(thingThatMoves) {
						thingThatMoves.style.animationPlayState = 'running';
						thingThatMoves.style.animationFillMode = 'forwards';
					});
				}
			}
		});
	},

	resetGoalList: function (siteURL, goalSelectId) {
		jQuery.ajax({
			url: siteURL + '/wp-admin/admin-ajax.php',
			data: {action: 'get_goals'},
			dataType: 'json',
			success: function (data) {
				if (data.error) {
					alert('Error contacting Patreon service: ' + data.error);
				} else {
					var select = jQuery('#' + goalSelectId);
					select.empty();
					select.append('<option value=0>Select Goal</option>');
					data.goals.forEach(function (goal) {
						select.append('<option value="' + goal.id + '">' + goal.attributes.title + '</option>');
					});
				}
			}
		});
	},
	
	buildStyleSheet: function(widget, widgetId, progressWidth) {
		var styleEl = document.getElementById('tracker-style-'+widgetId);
		var sheet = styleEl.sheet;
		
		var rules = sheet.cssRules;
		var keyframes = rules[0].cssRules;
		keyframes[1].style.setProperty('transform', 'translateX(' + progressWidth + 'px)');
		rules[1].style.setProperty('animation', 'tracker-tick-motion-' + widgetId + ' 1s cubic-bezier(0.215, 0.61, 0.355, 1)');

		return styleEl;
	}
};
