<?php

namespace GoalTrackerPatreon;

defined( 'ABSPATH' ) or die;

require_once(plugin_dir_path(__FILE__).'Patreon/API.php');
require_once(plugin_dir_path(__FILE__).'Patreon/OAuth.php');

class Creator {

	const invalid_token_message = 'invalid access token';
	private static $tokenRefreshErrorMessage;
	
	private $campaignData = false;

	public function __construct() {
		add_action('init', array($this, 'checkAccessToken'));

		add_action('wp_ajax_get_goal', array($this, 'printJSONGoal'));
		add_action('wp_ajax_nopriv_get_goal', array($this, 'printJSONGoal'));

		add_action('wp_ajax_get_goals', array($this, 'printJSONGoals'));
		add_action('wp_ajax_nopriv_get_goals', array($this, 'printJSONGoals'));

		add_action('add_option_'.Options::access_token_option_name, array($this, 'setCreatorId'));
		add_action('update_option_'.Options::access_token_option_name, array($this, 'setCreatorId'));
	}

	public function getCampaignData() {
		if (!$this->campaignData) {
			$creator_access_token = get_option(Options::access_token_option_name);
			$api_client = new API($creator_access_token);
			$this->campaignData = $api_client->fetch_campaign();
		}

		return $this->campaignData;
	}

	public function getCreatorId() {
		$creatorId = get_option(Options::creator_id_option_name, false);
		if (empty($creatorId) || $creatorId == self::invalid_token_message) {
			$rawData = $this->getCampaignData();
			if (is_array($rawData)) {
				foreach ($rawData['included'] as $item) {
					if ($item['type'] == 'user') {
						update_option(Options::creator_id_option_name, $item['id']);
					}
				}
			} else {
				update_option(Options::creator_id_option_name, self::invalid_token_message);
			}
		}
	}

	private function getGoal($goalId) {
		$rawData = $this->getCampaignData();

		$campaign = null;
		$user = null;
		$goal = null;
		$error = '';

		if (is_array($rawData)) {
			$campaign = $rawData['data'][0];

			foreach ($rawData['included'] as $item) {
				if ($item['type'] == 'user') {
					$user = $item;
				} else if ($item['type'] == 'goal') {
					if ($item['id'] == $goalId) {
						$goal = $item;
					}
				} else {
					$this->{$item['type'] . 's'}[] = $item;
				}
			}
		} else {
			$error = $rawData;
		}

		return array('campaign' => $campaign, 'creator' => $user, 'goal' => $goal, 'error' => $error);
	}

	private function getGoals() {
		$returnData = array('error' => '', 'goals' => array());

		$rawData = $this->getCampaignData();

		if (is_array($rawData)) {
			foreach ($rawData['included'] as $item) {
				if ($item['type'] == 'goal') {
					$returnData['goals'][] = $item;
				}
			}
			update_option(Options::available_goals_option_name, serialize($returnData['goals']));
		} else {
			$returnData['error'] = $rawData;
		}

		return $returnData;
	}

	public function printJSONGoal() {
		print json_encode($this->getGoal($_REQUEST['goalId']));
		die(); // yuck
	}

	public function printJSONGoals() {
		print json_encode($this->getGoals());
		die(); // yuck
	}

	public function setCreatorId() {
		$this->getCreatorId();
	}

	function checkAccessToken() {
		$expires = get_option(Options::access_token_expire_time_option_name, false);
		if ($expires === false || $expires < time() + 60*60*24) {
			$tokenStatus = $this->refreshAccessToken();
			if ($tokenStatus !== true) {
				self::$tokenRefreshErrorMessage = $tokenStatus;
				add_action('admin_notices', array($this, 'showTokenUpdateFailureAlert'));
			}
		}
	}

	private function refreshAccessToken() {
		$tokenStatus = true;

		$id = get_option(Options::client_id_option_name);
		$secret = get_option(Options::client_secret_option_name);
		$refreshToken = get_option(Options::refresh_token_option_name);

		$oauth = new OAuth($id, $secret);
		$response = $oauth->refresh_token($refreshToken, '');

		if (is_array($response)) {
			if (!isset($response['error'])) {
				update_option(Options::access_token_option_name, $response['access_token']);
				update_option(Options::refresh_token_option_name, $response['refresh_token']);
				update_option(Options::access_token_expire_time_option_name, \time() + $response['expires_in']);
			} else {
				$tokenStatus = $response['error'];
			}
		} else {
			$tokenStatus = $response;
		}
		
		return $tokenStatus;
	}
	
	public function showTokenUpdateFailureAlert() {
		$message = 'Goal Tracker for Patreon was unable to update your creator access token. The message returned from Patreon was '.self::$tokenRefreshErrorMessage;
		$html = <<<HTML
			<div class="notice update-nag">
				<p>$message</p>
			</div>
HTML;

		print $html;
	}
}

new Creator();
