<?php

namespace GoalTrackerPatreon;

defined( 'ABSPATH' ) or die;

class Goal_tracker_widget extends \WP_Widget {

	static $goals = array();

	public function __construct() {
		parent::__construct(
			'GoalTrackerForPatreonGoalWidget',
			'Patreon Goal Tracker',
			array( 'description' => __( 'Displays your progress toward a Patreon campaign goal.', 'text_domain' ), )
		);

		add_action('wp_enqueue_scripts', array($this, 'enqueueStuff'));
		add_action('admin_enqueue_scripts', array($this, 'enqueueAdminStuff'));
	}

	public function enqueueStuff() {
		wp_register_script('gtfp-goal-tracker-widget-js', plugins_url('js/widget.js', dirname(__FILE__)), array('jquery'));
		wp_enqueue_script('gtfp-goal-tracker-widget-js');

		$colorScheme = get_option(Options::display_color_scheme_option_name, 'dark');
		wp_register_style('gtfp-goal-tracker-widget-style', plugins_url('css/widget-'.$colorScheme.'.css', dirname(__FILE__)));
		wp_enqueue_style('gtfp-goal-tracker-widget-style');

		$inlineStyle = '';
		
		if (get_option(Options::display_text_color_option_name) === Options::option_value_custom) {
			$color = get_option(Options::display_text_custom_color_option_name);
			$inlineStyle .= "\n.thermometer text.gtfp { fill: $color; }";
		}
		
		if (get_option(Options::display_tick_color_option_name) === Options::option_value_custom) {
			$color = get_option(Options::display_tick_custom_color_option_name);
			$inlineStyle .= "\n.thermometer line.gtfp.tickmark { stroke: $color; }";
		}
		
		if (get_option(Options::display_progress_bar_color_option_name) === Options::option_value_custom) {
			$baseColor = get_option(Options::display_progress_bar_custom_color_option_name);
			$hilightColor = get_option(Options::display_progress_bar_custom_hilight_color_option_name);
			
			$inlineStyle .= 
				"
				.thermometer .gtfp-mg-stop-1 { stop-color: $baseColor; }
				.thermometer .gtfp-mg-stop-2 { stop-color: $hilightColor; }
				.thermometer .gtfp-mg-stop-3 { stop-color: $baseColor; }";
		}
	
		if (get_option(Options::display_progress_bar_color_option_name) === Options::option_value_custom) {
			$baseColor = get_option(Options::display_background_custom_color_option_name);
			$hilightColor = get_option(Options::display_background_custom_hilight_color_option_name);
			
			$inlineStyle .= 
				"
				.thermometer .gtfp-tbg-stop-1 { stop-color: $baseColor; }
				.thermometer .gtfp-tbg-stop-2 { stop-color: $hilightColor; }
				.thermometer .gtfp-tbg-stop-3 { stop-color: $baseColor; }";
		}
			
		if (strlen($inlineStyle) > 0) {	
			wp_add_inline_style('gtfp-goal-tracker-widget-style', $inlineStyle);
		}
	}
	
	public function enqueueAdminStuff() {
		wp_register_script('gtfp-goal-tracker-widget-js', plugins_url('js/widget.js', dirname(__FILE__)), array('jquery'));
		wp_enqueue_script('gtfp-goal-tracker-widget-js');
	}
	
	public function widget( $args, $instance ) {

		$siteURL = get_site_url();
		$goalId = empty($instance['goal_id']) ? 0 : $instance['goal_id'];
		$startCents = empty($instance['start']) ? 0 : $instance['start'];
		$startDollars = $startCents / 100;
		$reliableId = $args['widget_id'].'-interior';

		echo $args['before_widget'];
		echo "\n".'<div class="widget-interior" id="'.$reliableId.'">';
		if ( ! empty( $instance['title'] ) ) {
			echo $args['before_title'] . apply_filters( 'widget_title', $instance['title'] ). $args['after_title'];
		}

		if ( ! empty( $instance['pitch'] ) ) {
			echo '<div class="gtfp patreon-pitch-holder">'.$instance['pitch'].'</div>';
		}

		echo '<div class="gtfp patreon-campaign-info">';
		echo __( 'Contacting Patreon...', 'text_domain' ).'<br>&nbsp;';
		echo '</div>';
		
		// keyframes for the animation of this particular widget
		echo "\n\n";
		echo <<<STL
			<style id="tracker-style-$reliableId">
				@keyframes tracker-tick-motion-$reliableId {
					0% {transform: translateX(0px)}
					100% {transform: translateX(0px)}
				}

				#$reliableId g.tracker-tick {
					animation-play-state: paused;
				}
			</style>
STL;

		// TODO: clip path wasn't working when animations were applied, so an svg element is used for clipping instead. Should try to make clip path work.
		$svg = <<<SVG
			<svg xmlns="http://www.w3.org/2000/svg" width="200" height="58">
				<defs>
					<linearGradient id="Mercury-Gradient" x1="0" x2="0" y1="0" y2="1">
						<stop class="gtfp-mg-stop-1" offset="0%"/>
						<stop class="gtfp-mg-stop-2" offset="40%"/>
						<stop class="gtfp-mg-stop-3" offset="100%"/>
					</linearGradient>
					<linearGradient id="Thermometer-Background-Gradient" x1="0" x2="0" y1="0" y2="1">
						<stop class="gtfp-tbg-stop-1" offset="0%"/>
						<stop class="gtfp-tbg-stop-2" offset="40%"/>
						<stop class="gtfp-tbg-stop-3" offset="100%"/>
					</linearGradient>
				</defs>

				<g class="thermometer" transform="translate(20, 25)">
					<rect class="gtfp thermometer-background" x="-1" y="0" width="201" height="12" fill="url(#Thermometer-Background-Gradient)" />
					<svg xmlns="http://www.w3.org/2000/svg" x="0" width="2000" height="12">
						<g class="mercury" transform="scale(1,1)">
							<clipPath id="mercury-clip">
								<path d="M0,0 l1,0 l0,11 l-1,0 z" />
							</clipPath>
							<rect class="gtfp mercury" x="-1" y="1" width="1" height="10" fill="url(#Mercury-Gradient)" />
						</g>
					</svg>
					<!--<rect class="gtfp border-cap-right thermometer-background" x="200" y="0" width="1" height="12" />-->

					<g class="tick zero-tick" transform="translate(1, 13)">
						<line class="gtfp tickmark" x1="0" x2="0" y1="0" y2="4" />
						<text class="gtfp tick-label zero-tick-label" x="0" y="5" text-anchor="middle" font-size="10">
							<tspan dy="9">$startDollars</tspan>
						</text>
					</g>
					<g class="tick goal-tick" transform="translate(200, 13)">
						<line class="gtfp tickmark" x1="0" x2="0" y1="0" y2="4" />
						<text class="gtfp tick-label goal-tick-label" x="0" y="5" text-anchor="middle" font-size="10">
							<tspan dy="9">100</tspan>
						</text>
					</g>
					<g class="tick tracker-tick">
						<line class="gtfp tickmark" x1="0" x2="0" y1="0" y2="-5" />
						<line class="gtfp tickmark horizontal" x1="-7" x2="0" y1="-5" y2="-5" />
						<text class="gtfp tick-label goal-tick-label" x="-2" y="-7" text-anchor="end" font-size="10">
							<tspan>$startDollars</tspan>
						</text>
					</g>
				</g>
			</svg>
SVG;

		echo '<div class="gtfp thermometer">'.$svg.'</div>';

		echo "\n</div>";

		$js = "\ndocument.addEventListener('DOMContentLoaded', function() {";
		$js .= "\n\twindow.patreonCampaignTracker.loadGoal('$siteURL', '$reliableId', '$goalId', '$startCents');";
		$js .= "\n});";
		
		if (get_option(Options::display_text_color_option_name) === Options::option_value_inherit) {
			$js .= "\nwindow.patreonCampaignTracker.setTextColorFromWidget('$reliableId');";
		}

		if (get_option(Options::display_tick_color_option_name) === Options::option_value_inherit) {
			$js .= "\nwindow.patreonCampaignTracker.setTickColorFromText('$reliableId');";
		}
		
		echo "<script type=\"text/javascript\">$js</script>";

		echo $args['after_widget'];
	}

	public function form( $instance ) {
		$title = ! empty( $instance['title'] ) ? $instance['title'] : __( 'New title', 'text_domain' );
		$pitch = ! empty( $instance['pitch'] ) ? $instance['pitch'] : '';
		$goal_id = ! empty( $instance['goal_id'] ) ? $instance['goal_id'] : 0;
		$start = isset($instance['start']) && is_numeric($instance['start']) ? $instance['start'] : 0;

		echo '<p>';
		echo '<label for="'.$this->get_field_id( 'title' ).'">'._e( 'Title:' ).'</label>';
		echo '<input class="widefat" id="'.$this->get_field_id( 'title' ).'" name="'.$this->get_field_name( 'title' ).'" type="text" value="'.esc_attr( $title ).'">';
		echo '</p>';

		$siteURL = get_site_url();
		$goalSelectId = $this->get_field_id( 'goal_id' );
		$goalSelectName = $this->get_field_name('goal_id');
		$currentGoalId = esc_attr( $goal_id );

		echo '<p>';
		echo '<label for="'.$this->get_field_id( 'goal_id' ).'">'._e( 'Goal:' ).'</label>';
		echo '<select class="widefat" id="'.$goalSelectId.'" name="'.$goalSelectName.'" value="'.$currentGoalId.'">';

		$availableGoals = unserialize(get_option(Options::available_goals_option_name));
		echo '<option value="0">Select Goal</option>';
		if (!empty($availableGoals)) {
			foreach ($availableGoals as $goal) {
				echo '<option value="' . $goal['id'] . '"' . ($currentGoalId == $goal['id'] ? ' selected' : '') . '>' . $goal['attributes']['title'] . '</option>';
			}
		}

		echo '</select>';
		echo '</p>';

		$resetButtonId = $this->get_field_id( 'goal_reset' );
		echo '<input type="button" value="Reset Goal List" id="'.$resetButtonId.'" style="float:left; margin-right: 1em">';

		$js = <<<JS
		jQuery('#$resetButtonId').click(function() {
			window.patreonCampaignTracker.resetGoalList('$siteURL', '$goalSelectId');
		});
JS;

		echo '<script type="text/javascript">'.$js.'</script>';
		echo '<p style="font-size:smaller;">When you change your goals on the Patreon site, click this button to reload your goals here.</p>';

		echo '<p>';
		echo '<label for="'.$this->get_field_id( 'pitch' ).'">'._e( 'Pitch:' ).'</label>';
		echo '<textarea class="widefat" id="'.$this->get_field_id( 'pitch' ).'" name="'.$this->get_field_name( 'pitch' ).'">'.esc_attr( $pitch ).'</textarea>';
		echo '<br><span style="font-size: smaller;">A short blurb telling people how awesome the goal is. You can leave this blank if you want.</span>';
		echo '</p>';
		
		echo '<p>';
		echo '<label for="'.$this->get_field_id( 'start' ).'">'._e( 'Start (in cents):' ).'</label>';
		echo '<input class="widefat" id="'.$this->get_field_id( 'start' ).'" name="'.$this->get_field_name( 'start' ).'" type="text" value="'.esc_attr( $start ).'">';
		echo '<br><span style="font-size: smaller;">If you have incremental goals, you can set the start of your progress meter to something besides 0.</span>';
		echo '</p>';
	}

	public function update( $new_instance, $old_instance ) {
		$instance = array();
		$instance['title'] = ( ! empty( $new_instance['title'] ) ) ? strip_tags( $new_instance['title'] ) : '';
		$instance['pitch'] = ( ! empty( $new_instance['pitch'] ) ) ? strip_tags( $new_instance['pitch'] ) : '';
		$instance['goal_id'] = ( isset($new_instance['goal_id']) && is_numeric($new_instance['goal_id']) ) ? strip_tags( $new_instance['goal_id'] ) : '';
		$instance['start'] = ( isset($new_instance['start']) && is_numeric($new_instance['start']) ) ? strip_tags( $new_instance['start'] ) : '';

		return $instance;
	}
}

